<?php

namespace Drupal\open_readspeaker\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The setting form for the open_readspeaker settings.
 *
 * @package Drupal\open_readspeaker\Form
 */
class OpenReadSpeakerSettingsForm extends ConfigFormBase {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a PerformanceForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    ModuleHandlerInterface $module_handler,
    RendererInterface $renderer) {
    parent::__construct($config_factory);

    $this->moduleHandler = $module_handler;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('module_handler'),
      $container->get('renderer'),
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'open_readspeaker.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'open_read_speaker_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('open_readspeaker.settings');

    $language_options = [
      'none'  => $this->t('Select a language'),
      'ar_ar' => $this->t('@arabic - Arabic', ['@arabic' => urldecode('%D8%B9%D8%B1%D8%A8%D9%8A')]),
      'ca_es' => $this->t('Català - Catalan'),
      'cs_cz' => $this->t('Čeština - Czech'),
      'cy_cy' => $this->t('Cymraeg - Welsh'),
      'da_dk' => $this->t('Dansk - Danish'),
      'de_de' => $this->t('Deutsch - German'),
      'en_in' => $this->t('English (Indian) - English'),
      'en_us' => $this->t('English (US) - English'),
      'en_uk' => $this->t('English (British) - English'),
      'en_au' => $this->t('English (Australia) - English'),
      'en_sc' => $this->t('English (Scottish) - English'),
      'en_za' => $this->t('English (South African) - English'),
      'es_ar' => $this->t('Español (de AR) - Spanish'),
      'es_co' => $this->t('Español (de Colombia) - Spanish'),
      'es_es' => $this->t('Español (de España) - Spanish'),
      'es_mx' => $this->t('Español (de Mexico) - Spanish'),
      'es_us' => $this->t('Español (de US) - Spanish'),
      'eu_es' => $this->t('Euskara - Basque'),
      'el_gr' => $this->t('Ελληνικά - Greek'),
      'fa_ir' => $this->t('فارسی - Farsi'),
      'fo_fo' => $this->t('Føroyskt - Faroese'),
      'fi_fi' => $this->t('Suomi - Finnish'),
      'fr_fr' => $this->t('Français - French'),
      'fr_be' => $this->t('Français (Belge) - French (Belgian)'),
      'fr_ca' => $this->t('Français (Canadian) - French (Canadian)'),
      'fy_nl' => $this->t('Frysk - Frisian'),
      'gl_es' => $this->t('Galego - Galician'),
      'he_il' => $this->t('עִברִית - Hebrew'),
      'hi_in' => $this->t('@hindi - Hindi', ['@hindi' => urldecode('%E0%A4%B9%E0%A4%BF%E0%A4%A8%E0%A5%8D%E0%A4%A6%E0%A5%80%20(%E0%A4%AD%E0%A4%BE%E0%A4%B0%E0%A4%A4)')]),
      'hr_hr' => $this->t('Hrvatski - Croatian'),
      'id_id' => $this->t('Bahasa Indonesia - Indonesian'),
      'it_it' => $this->t('Italiano - Italian'),
      'is_is' => $this->t('íslenska - Icelandic'),
      'ja_jp' => $this->t('日本語 - Japanese'),
      'ko_kr' => $this->t('한국어 - Korean'),
      'lv_lv' => $this->t('Latviski - Latvian'),
      'pl_pl' => $this->t('Polski - Polish'),
      'pt_br' => $this->t('Português (Brasil) - Portuguese'),
      'pt_pt' => $this->t('Português (Europeu) - Portuguese'),
      'ro_ro' => $this->t('Română - Romanian'),
      'nl_nl' => $this->t('Nederlands - Dutch'),
      'nl_be' => $this->t('Nederlands (Belgisch) - Dutch (Belgian)'),
      'no_nb' => $this->t('Bokmål - Norwegian'),
      'no_nn' => $this->t('Nynorsk - Norwegian'),
      'ru_ru' => $this->t('Русский - Russian'),
      'sk_sk' => $this->t('Slovenčina - Slovak'),
      'sv_se' => $this->t('Svenska - Swedish'),
      'th_th' => $this->t('@thai - Thai', ['@thai' => urldecode('%E0%B9%84%E0%B8%97%E0%B8%A2')]),
      'tr_tr' => $this->t('Türkçe - Turkish'),
      'uk_ua' => $this->t('Українська - Ukrainian'),
      'zh_cn' => $this->t('@mandarin - Mandarin Chinese', ['@mandarin' => urldecode('%E4%B8%AD%E6%96%87%20(%E7%AE%80%E4%BD%93)')]),
      'zh_hk' => $this->t('@traditional - Traditional Chinese', ['@traditional' => urldecode('%E4%B8%AD%E5%9C%8B%EF%BC%88%E7%B9%81%E9%AB%94%EF%BC%89')]),
      'zh_tw' => $this->t('@taiwanese - Taiwanese Chinese', ['@taiwanese' => urldecode('%E4%B8%AD%E6%96%87+%28%E8%87%BA%E7%81%A3%E5%9C%8B%E8%AA%9E%29')]),
    ];

    $translationOptions = [
      'ar_ar' => $this->t('@arabic - Arabic', ['@arabic' => urldecode('%D8%B9%D8%B1%D8%A8%D9%8A')]),
      'da_dk' => $this->t('Dansk - Danish'),
      'de_de' => $this->t('Deutsch - German'),
      'el_gr' => $this->t('Ελληνικά - Greek'),
      'en' => $this->t('English - English'),
      'es_es' => $this->t('Español - Spanish'),
      'fi_fi' => $this->t('Suomi - Finnish'),
      'fr_fr' => $this->t('Français - French'),
      'hu_hu' => $this->t('Magyar - Hungarian'),
      'id_id' => $this->t('Bahasa Indonesia - Indonesian'),
      'it_it' => $this->t('Italiano - Italian'),
      'ja_jp' => $this->t('日本語 - Japanese'),
      'ko_kr' => $this->t('한국어 - Korean'),
      'lv_lv' => $this->t('Latviski - Latvian'),
      'nl_nl' => $this->t('Nederlands - Dutch'),
      'no_nb' => $this->t('Bokmål - Norwegian'),
      'pl_pl' => $this->t('Polski - Polish'),
      'pt_br' => $this->t('Português (Brasil) - Portuguese'),
      'pt_pt' => $this->t('Português (Europeu) - Portuguese'),
      'ro_ro' => $this->t('Română - Romanian'),
      'ru_ru' => $this->t('Русский - Russian'),
      'sv_se' => $this->t('Svenska - Swedish'),
      'th_th' => $this->t('@thai - Thai', ['@thai' => urldecode('%E0%B9%84%E0%B8%97%E0%B8%A2')]),
      'tr_tr' => $this->t('Türkçe - Turkish'),
      'uk_ua' => $this->t('Українська - Ukrainian'),
      'vi_vn' => $this->t('Tiếng Việt - Vietnamese'),
      'zh_cn' => $this->t('@mandarin - Mandarin Chinese', ['@mandarin' => urldecode('%E4%B8%AD%E6%96%87%20(%E7%AE%80%E4%BD%93)')])
    ];

    $cdnRegionOptions = [
      'eu' => $this->t('Europe'),
      'me' => $this->t('Middle East'),
      'af' => $this->t('Africa'),
      'as' => $this->t('Asia'),
      'oc' => $this->t('Oceania'),
      'eas' => $this->t('East Asia'),
      'na' => $this->t('North America'),
      'sa' => $this->t('South America'),
    ];

    $form['global'] = [
      '#title' => $this->t('General settings for ReadSpeaker'),
      '#type' => 'fieldset',
    ];

    $form['global']['open_readspeaker_accountid'] = [
      '#title' => $this->t('Account ID'),
      '#description' => $this->t('Enter your ReadSpeaker account ID'),
      '#type' => 'textfield',
      '#default_value' => $config->get('open_readspeaker_accountid'),
      '#required' => TRUE,
    ];

    $form['global']['open_readspeaker_cdn_region'] = [
      '#title' => $this->t('CDN'),
      '#description' => $this->t('Select the CDN for your region'),
      '#type' => 'select',
      '#options' => $cdnRegionOptions,
      '#default_value' => $config->get('open_readspeaker_cdn_region'),
      '#required' => TRUE,
    ];

    $form['global']['open_readspeaker_i18n'] = [
      '#title' => $this->t('Language'),
      '#description' => $this->t('Select which language your ReadSpeaker account supports.'),
      '#type' => 'select',
      '#options' => $language_options,
      '#default_value' => $config->get('open_readspeaker_i18n'),
      '#required' => TRUE,
    ];

    $form['global']['open_readspeaker_voice'] = [
      '#title' => $this->t('Voice'),
      '#description' => $this->t('Enter male or female language voice name. English language examples: Alice or Hugh. Only relevant in case your account has more than one voice per language. Find available voices under Customer Portal > "Voices"'),
      '#type' => 'textfield',
      '#default_value' => $config->get('open_readspeaker_voice'),
    ];

    if ($this->moduleHandler->moduleExists('token')) {
      $form['global']['token_browser'] = [
        '#theme' => 'token_tree_link',
        '#token_types' => [
          'open_readspeaker',
        ],
      ];
      $tokenBrowserLink = $this->renderer->render($form['global']['token_browser']);
    }

    $form['global']['open_readspeaker_url'] = [
      '#title' => $this->t('URL'),
      '#description' => $this->t('The remote URL of the ReadSpeaker JavaScript.') . ($tokenBrowserLink ?? ''),
      '#type' => 'textfield',
      '#default_value' => $config->get('open_readspeaker_url'),
      '#required' => TRUE,
    ];

    $form['tabs'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('ReadSpeaker widget configuration'),
      '#attached' => [
        'library' => [
          'open_readspeaker/open_readspeaker.admin',
        ],
      ],
    ];

    $rsConf = $config->get('rsConf');

    // General specific configurations.
    $form['general'] = [
      '#type' => 'details',
      '#title' => $this->t('General'),
      '#group' => 'tabs',
      '#weight' => -1,
    ];

    $generalConfig = $rsConf['general'];

    $form['general']['confirmPolicy'] = [
      '#title' => $this->t('Confirm policy'),
      '#description' => $this->t('When set, the user will need to read and confirm a message before the playback can start on first click.'),
      '#type' => 'textfield',
      '#default_value' => $generalConfig['confirmPolicy']
    ];

    $form['general']['cookieLifetime'] = [
      '#title' => $this->t('Cookie lifetime'),
      '#description' => $this->t('How long the settings cookie should survive on the user’s computer, expressed in milliseconds.'),
      '#type' => 'number',
      '#min' => -1,
      '#default_value' => $generalConfig['cookieLifetime']
    ];

    $form['general']['cookieName'] = [
      '#title' => $this->t('Cookie name'),
      '#description' => $this->t('The name of the main cookie that will be used to store users’ personalized settings.'),
      '#type' => 'textfield',
      '#default_value' => $generalConfig['cookieName']
    ];

    $form['general']['customTransLangs'] = [
      '#title' => $this->t('Translation languages'),
      '#description' => $this->t('Overrides the default target languages list.'),
      '#type' => 'checkboxes',
      '#options' => $translationOptions,
      '#default_value' => $generalConfig['customTransLangs']
    ];

    $form['general']['defaultSpeedValue'] = [
      '#title' => $this->t('Default speed value'),
      '#description' => $this->t('This is the default speed value sent to ReadSpeaker.'),
      '#type' => 'number',
      '#min' => 0,
      '#default_value' => $generalConfig['defaultSpeedValue']
    ];

    $form['general']['domain'] = [
      '#title' => $this->t('Domain'),
      '#description' => $this->t('The domain that is used for the calls to the ReadSpeaker servers.'),
      '#type' => 'textfield',
      '#default_value' => $generalConfig['domain']
    ];

    $form['general']['nativeLanguages'] = [
      '#title' => $this->t('Native languages'),
      '#description' => $this->t('The languages in Reading Voice and Translation can be displayed as their own localized names.'),
      '#type' => 'checkbox',
      '#default_value' => $generalConfig['nativeLanguages']
    ];

    $form['general']['popupCloseTime'] = [
      '#title' => $this->t('Popup close time'),
      '#description' => $this->t('Determines the amount of time the popup will be visible when the user has selected some text. Expressed in milliseconds.'),
      '#type' => 'number',
      '#min' => 0,
      '#default_value' => $generalConfig['popupCloseTime']
    ];

    $form['general']['shadowDomSelector'] = [
      '#title' => $this->t('Shadow DOM selector'),
      '#description' => $this->t('Enable reading content from a shadow DOM.'),
      '#type' => 'textfield',
      '#default_value' => $generalConfig['shadowDomSelector']
    ];

    $form['general']['subdomain'] = [
      '#title' => $this->t('Subdomain'),
      '#description' => $this->t('The sub-domain that is used for the calls to the ReadSpeaker servers.'),
      '#type' => 'textfield',
      '#default_value' => $generalConfig['subdomain']
    ];

    $form['general']['syncContainer'] = [
      '#title' => $this->t('Sync container'),
      '#description' => $this->t('Sets the type of element the sync tags will use.'),
      '#type' => 'textfield',
      '#default_value' => $generalConfig['syncContainer']
    ];

    $form['general']['saveLangVoice'] = [
      '#title' => $this->t('Save language voice'),
      '#description' => $this->t('The language selected in the voice settings tool will be stored in a cookie.'),
      '#type' => 'checkbox',
      '#default_value' => $generalConfig['saveLangVoice'],
    ];

    $form['general']['translatedDisclaimer'] = [
      '#title' => $this->t('Translated disclaimer'),
      '#description' => $this->t('The translation disclaimer can be displayed in the target language.'),
      '#type' => 'checkbox',
      '#default_value' => $generalConfig['translatedDisclaimer'],
    ];

    $form['general']['skipHiddenContent'] = [
      '#title' => $this->t('Skip hidden content'),
      '#description' => $this->t('By using POST mode and Skip hidden content option, any non-visible content will be ignored at reading. This works for elements with CSS style display:none, with CSS style visibility:hidden or with HTML hidden attribute.'),
      '#type' => 'checkbox',
      '#default_value' => $generalConfig['skipHiddenContent'],
    ];

    $ignoreAttributes = array_map('trim', explode(',', $generalConfig['labels']['ignoreSelector']));
    $labelsIgnoreSelector = [];
    foreach ($ignoreAttributes as $attribute) {
      $match = preg_match('/[\w-]+/', $attribute, $matches);
      if ($match) {
        $labelsIgnoreSelector[] = $matches[0];
      }
    }

    $form['general']['labelsIgnoreSelector'] = [
      '#title' => $this->t('Ignore selector'),
      '#description' => $this->t('Skip html attribute values from being read, e.g <em>aria-label</em>. You can add multiple comma separated values.'),
      '#type' => 'textfield',
      '#default_value' => join(', ', $labelsIgnoreSelector),
    ];

    $form['general']['usePost'] = [
      '#title' => $this->t('Use POST mode'),
      '#description' => $this->t('Enabling this option will POST the html/text to the ReadSpeaker service instead of ReadSpeaker accessing your site, which will make the service work behind password protected pages.'),
      '#type' => 'checkbox',
      '#default_value' => $generalConfig['usePost'],
      '#states' => [
        'checked' => [':input[name="skipHiddenContent"]' => ['checked' => TRUE],]
      ],
    ];

    // Settings specific configurations.
    $form['settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Settings'),
      '#group' => 'tabs',
    ];

    $settingsConfig = $rsConf['settings'];

    $form['settings']['hl'] = [
      '#title' => $this->t('Highlight style'),
      '#description' => $this->t('Which highlighting style to use.'),
      '#type' => 'select',
      '#options' => [
        'wordsent' => $this->t('Word & sentences'),
        'word' => $this->t('Word'),
        'sent' => $this->t('Sentences')
      ],
      '#default_value' => $settingsConfig['hl']
    ];

    $form['settings']['hlicon'] = [
      '#title' => $this->t('Highlight icon'),
      '#description' => $this->t('Whether or not to display a popup button when selecting text on a web page.'),
      '#type' => 'select',
      '#options' => [
        'iconon' => $this->t('On'),
        'iconoff' => $this->t('Off')
      ],
      '#default_value' => $settingsConfig['hlicon']
    ];

    $form['settings']['hlscroll'] = [
      '#title' => $this->t('Highlight scroll'),
      '#description' => $this->t('Whether or not to automatically scroll the page vertically when the reading has reached the bottom of the browser’s viewport.'),
      '#type' => 'select',
      '#options' => [
        'scrollon' => $this->t('On'),
        'scrolloff' => $this->t('Off')
      ],
      '#default_value' => $settingsConfig['hlscroll']
    ];

    $form['settings']['hlsent'] = [
      '#title' => $this->t('Highlighting sentences'),
      '#description' => $this->t('The color that will be used for highlighting sentences, expressed as a valid CSS color value.'),
      '#type' => 'color',
      '#default_value' => $settingsConfig['hlsent']
    ];

    $form['settings']['hlspeed'] = [
      '#title' => $this->t('Highlighting speed'),
      '#description' => $this->t('The reading speed.'),
      '#type' => 'select',
      '#options' => [
        'slow' => $this->t('Slow'),
        'medium' => $this->t('Medium'),
        'fast' => $this->t('Fast')
      ],
      '#default_value' => $settingsConfig['hlspeed']
    ];

    $form['settings']['hltext'] = [
      '#title' => $this->t('Highlighting text'),
      '#description' => $this->t('The color that will be used for the text in the highlighted elements, expressed as a valid CSS color value.'),
      '#type' => 'color',
      '#default_value' => $settingsConfig['hltext']
    ];

    $form['settings']['hltoggle'] = [
      '#title' => $this->t('Highlighting toggle'),
      '#description' => $this->t('This setting toggles highlighting on or off (hlon or hloff).'),
      '#type' => 'select',
      '#options' => [
        'hlon' => $this->t('On'),
        'hloff' => $this->t('Off')
      ],
      '#default_value' => $settingsConfig['hltoggle']
    ];

    $form['settings']['hlword'] = [
      '#title' => $this->t('Highlighting words'),
      '#description' => $this->t('The color that will be used for highlighting words, expressed as a valid CSS color value.'),
      '#type' => 'color',
      '#default_value' => $settingsConfig['hlword']
    ];

    // Keyboard shortcuts specific configurations.
    $form['keyboard'] = [
      '#type' => 'details',
      '#title' => $this->t('Keyboard combinations'),
      '#group' => 'tabs',
    ];

    $form['keyboard']['kbClicklisten'] = [
      '#title' => $this->t('Click listen'),
      '#description' => $this->t('Keyboard combination for \'clicklisten\'.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['clicklisten']
    ];

    $form['keyboard']['kbControlpanel'] = [
      '#title' => $this->t('Control panel'),
      '#description' => $this->t('Keyboard combination for \'controlpanel\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['controlpanel']
    ];

    $form['keyboard']['kbDictionary'] = [
      '#title' => $this->t('Dictionary'),
      '#description' => $this->t('Keyboard combination for \'dictionary\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['dictionary']
    ];

    $form['keyboard']['kbDownload'] = [
      '#title' => $this->t('Download'),
      '#description' => $this->t('Keyboard combination for \'download\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['download']
    ];

    $form['keyboard']['kbEnlarge'] = [
      '#title' => $this->t('Enlarge'),
      '#description' => $this->t('Keyboard combination for \'enlarge\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['enlarge']
    ];

    $form['keyboard']['kbFontsizeminus'] = [
      '#title' => $this->t('Decrease font size'),
      '#description' => $this->t('Keyboard combination for \'fontsizeminus\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['fontsizeminus']
    ];

    $form['keyboard']['kbFontsizeplus'] = [
      '#title' => $this->t('Increase font size'),
      '#description' => $this->t('Keyboard combination for \'fontsizeplus\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['fontsizeplus']
    ];

    $form['keyboard']['kbFormreading'] = [
      '#title' => $this->t('Form reading'),
      '#description' => $this->t('Keyboard combination for \'formreading\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['formreading']
    ];

    $form['keyboard']['kbHelp'] = [
      '#title' => $this->t('Help'),
      '#description' => $this->t('Keyboard combination for \'help\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['help']
    ];

    $form['keyboard']['kbMenu'] = [
      '#title' => $this->t('Menu'),
      '#description' => $this->t('Keyboard combination for \'menu\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['menu']
    ];

    $form['keyboard']['kbPagemask'] = [
      '#title' => $this->t('Page mask'),
      '#description' => $this->t('Keyboard combination for \'pagemask\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['pagemask']
    ];

    $form['keyboard']['kbPause'] = [
      '#title' => $this->t('Pause'),
      '#description' => $this->t('Keyboard combination for \'pause\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['pause']
    ];

    $form['keyboard']['kbPlay'] = [
      '#title' => $this->t('Play'),
      '#description' => $this->t('Keyboard combination for \'play\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['play']
    ];

    $form['keyboard']['kbPlayerfocus'] = [
      '#title' => $this->t('Player focus'),
      '#description' => $this->t('Keyboard combination for \'playerfocus\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['playerfocus']
    ];

    $form['keyboard']['kbSettings'] = [
      '#title' => $this->t('Settings'),
      '#description' => $this->t('Keyboard combination for \'settings\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['settings']
    ];

    $form['keyboard']['kbStop'] = [
      '#title' => $this->t('Stop'),
      '#description' => $this->t('Keyboard combination for \'stop\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['stop']
    ];

    $form['keyboard']['kbTextmode'] = [
      '#title' => $this->t('Text mode'),
      '#description' => $this->t('Keyboard combination for \'textmode\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['textmode']
    ];

    $form['keyboard']['kbTranslation'] = [
      '#title' => $this->t('Translation'),
      '#description' => $this->t('Keyboard combination for \'translation\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['translation']
    ];

    $form['keyboard']['kbReadingvoice'] = [
      '#title' => $this->t('Reading voice'),
      '#description' => $this->t('Keyboard combination for \'readingvoice\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['readingvoice']
    ];

    $form['keyboard']['kbDetachfocus'] = [
      '#title' => $this->t('Detach focus'),
      '#description' => $this->t('Keyboard combination for \'detachfocus\' action.'),
      '#type' => 'textfield',
      '#default_value' => $settingsConfig['kb']['detachfocus']
    ];

    // UI specific configurations.
    $form['ui'] = [
      '#type' => 'details',
      '#title' => $this->t('User interface'),
      '#group' => 'tabs',
    ];

    $uiConfig = $rsConf['ui'];

    $form['ui']['mobileVertPos'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Mobile vertical position')
    ];

    [$mobileVertAlign, $mobileVertOffset] = explode('=', $uiConfig['mobileVertPos']);

    $form['ui']['mobileVertPos']['vertAlign'] = [
      '#title' => $this->t('Alignment'),
      '#description' => $this->t('The vertical alignment of the mobile menu.'),
      '#type' => 'radios',
      '#options' => [
        'top' => $this->t('Top'),
        'bottom' => $this->t('Bottom')
      ],
      '#default_value' => $mobileVertAlign,
      '#required' => TRUE
    ];

    $form['ui']['mobileVertPos']['vertOffset'] = [
      '#title' => $this->t('Offset'),
      '#description' => $this->t('The vertical offset of the mobile menu.'),
      '#type' => 'number',
      '#min' => 0,
      '#default_value' => $mobileVertOffset,
      '#required' => TRUE
    ];

    $form['ui']['controlpanelVertical'] = [
      '#title' => $this->t('Default vertical position'),
      '#description' => $this->t('Change the default vertical alignment.'),
      '#type' => 'select',
      '#options' => [
        'top' => $this->t('Top'),
        'bottom' => $this->t('Bottom')
      ],
      '#default_value' => $uiConfig['controlpanel']['vertical']
    ];

    $form['ui']['controlpanelHorizontal'] = [
      '#title' => $this->t('Default horizontal position'),
      '#description' => $this->t('Change the default the horizontal alignment.'),
      '#type' => 'select',
      '#options' => [
        'left' => $this->t('Left'),
        'right' => $this->t('Right')
      ],
      '#default_value' => $uiConfig['controlpanel']['horizontal']
    ];

    $form['ui']['tools'] = [
      '#title' => $this->t('Tools'),
      '#description' => $this->t('Enable / Disable specific tool menu entries.'),
      '#type' => 'checkboxes',
      '#options' => [
        'settings' => $this->t('Settings'),
        'voicesettings' => $this->t('Voice settings'),
        'clicklisten' => $this->t('Click listen'),
        'enlarge' => $this->t('Enlarge'),
        'formreading' => $this->t('Form reading'),
        'textmode' => $this->t('Text mode'),
        'pagemask' => $this->t('Page mask'),
        'download' => $this->t('Download'),
        'help' => $this->t('Help'),
        'dictionary' => $this->t('Dictionary'),
        'translation' => $this->t('Translation'),
        'skipbuttons' => $this->t('Skip buttons'),
        'speedbutton' => $this->t('Speed button'),
        'controlpanel' => $this->t('Control panel')
      ],
      '#default_value' => array_keys(array_filter($uiConfig['tools'])),
    ];

    // TODO implement 'phrases' setting

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $ignoreAttributes = array_map('trim', explode(',', $form_state->getValue('labelsIgnoreSelector')));
    $validAttributes = [];
    foreach ($ignoreAttributes as $key => $attribute) {
      $match = preg_match('/^[\w-]+$/', $attribute);
      if ($match) {
        $validAttributes[] = $key;
      }
    }

    $errorAttributes = array_diff_key($ignoreAttributes, $validAttributes);

    if ($errorAttributes) {
      $form_state->setErrorByName('labelsIgnoreSelector', $this->t('The attributes %attributes must only match word characters and dashes.', [
        '%attributes' => join(', ', $errorAttributes)
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $mobileVertPos = $form_state->getValue('vertAlign') . '=' . $form_state->getValue('vertOffset');
    $ignoreSelector = explode(',', $form_state->getValue('labelsIgnoreSelector'));
    $joinedIgnoreSelector = join(', ', array_map(fn($selector) => '[' . trim($selector) . ']', $ignoreSelector));

    $this->config('open_readspeaker.settings')
      ->set('open_readspeaker_accountid', $form_state->getValue('open_readspeaker_accountid'))
      ->set('open_readspeaker_cdn_region', $form_state->getValue('open_readspeaker_cdn_region'))
      ->set('open_readspeaker_i18n', $form_state->getValue('open_readspeaker_i18n'))
      ->set('open_readspeaker_voice', $form_state->getValue('open_readspeaker_voice'))
      ->set('open_readspeaker_url', $form_state->getValue('open_readspeaker_url'))
      ->set('rsConf.general.confirmPolicy', $form_state->getValue('confirmPolicy'))
      ->set('rsConf.general.cookieLifetime', $form_state->getValue('cookieLifetime'))
      ->set('rsConf.general.cookieName', $form_state->getValue('cookieName'))
      ->set('rsConf.general.customTransLangs', array_values(array_filter($form_state->getValue('customTransLangs'))))
      ->set('rsConf.general.defaultSpeedValue', $form_state->getValue('defaultSpeedValue'))
      ->set('rsConf.general.domain', $form_state->getValue('domain'))
      ->set('rsConf.general.nativeLanguages', $form_state->getValue('nativeLanguages'))
      ->set('rsConf.general.popupCloseTime', $form_state->getValue('popupCloseTime'))
      ->set('rsConf.general.shadowDomSelector', $form_state->getValue('shadowDomSelector'))
      ->set('rsConf.general.subdomain', $form_state->getValue('subdomain'))
      ->set('rsConf.general.syncContainer', $form_state->getValue('syncContainer'))
      ->set('rsConf.general.saveLangVoice', $form_state->getValue('saveLangVoice'))
      ->set('rsConf.general.translatedDisclaimer', $form_state->getValue('translatedDisclaimer'))
      ->set('rsConf.general.skipHiddenContent', $form_state->getValue('skipHiddenContent'))
      ->set('rsConf.general.labels.ignoreSelector', $joinedIgnoreSelector)
      ->set('rsConf.general.usePost', $form_state->getValue('usePost'))
      ->set('rsConf.settings.hl', $form_state->getValue('hl'))
      ->set('rsConf.settings.hlicon', $form_state->getValue('hlicon'))
      ->set('rsConf.settings.hlscroll', $form_state->getValue('hlscroll'))
      ->set('rsConf.settings.hlsent', $form_state->getValue('hlsent'))
      ->set('rsConf.settings.hlspeed', $form_state->getValue('hlspeed'))
      ->set('rsConf.settings.hltext', $form_state->getValue('hltext'))
      ->set('rsConf.settings.hltoggle', $form_state->getValue('hltoggle'))
      ->set('rsConf.settings.hlword', $form_state->getValue('hlword'))
      ->set('rsConf.settings.kb.clicklisten', $form_state->getValue('kbClicklisten'))
      ->set('rsConf.settings.kb.controlpanel', $form_state->getValue('kbControlpanel'))
      ->set('rsConf.settings.kb.dictionary', $form_state->getValue('kbDictionary'))
      ->set('rsConf.settings.kb.download', $form_state->getValue('kbDownload'))
      ->set('rsConf.settings.kb.enlarge', $form_state->getValue('kbEnlarge'))
      ->set('rsConf.settings.kb.fontsizeminus', $form_state->getValue('kbFontsizeminus'))
      ->set('rsConf.settings.kb.fontsizeplus', $form_state->getValue('kbFontsizeplus'))
      ->set('rsConf.settings.kb.formreading', $form_state->getValue('kbFormreading'))
      ->set('rsConf.settings.kb.help', $form_state->getValue('kbHelp'))
      ->set('rsConf.settings.kb.menu', $form_state->getValue('kbMenu'))
      ->set('rsConf.settings.kb.pagemask', $form_state->getValue('kbPagemask'))
      ->set('rsConf.settings.kb.pause', $form_state->getValue('kbPause'))
      ->set('rsConf.settings.kb.play', $form_state->getValue('kbPlay'))
      ->set('rsConf.settings.kb.playerfocus', $form_state->getValue('kbPlayerfocus'))
      ->set('rsConf.settings.kb.settings', $form_state->getValue('kbSettings'))
      ->set('rsConf.settings.kb.stop', $form_state->getValue('kbStop'))
      ->set('rsConf.settings.kb.textmode', $form_state->getValue('kbTextmode'))
      ->set('rsConf.settings.kb.translation', $form_state->getValue('kbTranslation'))
      ->set('rsConf.settings.kb.readingvoice', $form_state->getValue('kbReadingvoice'))
      ->set('rsConf.settings.kb.detachfocus', $form_state->getValue('kbDetachfocus'))
      ->set('rsConf.ui.mobileVertPos', $mobileVertPos)
      ->set('rsConf.ui.controlpanel.vertical', $form_state->getValue('controlpanelVertical'))
      ->set('rsConf.ui.controlpanel.horizontal', $form_state->getValue('controlpanelHorizontal'))
      ->set('rsConf.ui.tools', $form_state->getValue('tools'))
      ->save();
  }

}
